package at.ac.uibk.gitsearch.web.rest;

import java.io.IOException;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import at.ac.uibk.gitsearch.es.model.DocumentInfo;
import at.ac.uibk.gitsearch.service.SearchService;
import at.ac.uibk.gitsearch.service.dto.SearchInputDTO;
import at.ac.uibk.gitsearch.service.dto.SearchResultsDTO;

/**
 * REST controller for managing {@link DocumentInfo}.
 */
@RestController
@RequestMapping("/api")
@Transactional
public class SearchResource {

	private final Logger log = LoggerFactory.getLogger(SearchResource.class);

    private final SearchService searchService;

    public SearchResource(SearchService searchService) {
        this.searchService = searchService;
    }

    /**
      * {@code SEARCH  /search/page-details} : search for the searchResults corresponding
     *  to the query.
     *
	 * @param query the query of the searchResult search.
	 * @return the result of the search.
     */
    @PostMapping("/search/page-details")
    public SearchResultsDTO
    searchPageDetails(@RequestBody SearchInputDTO query) throws IOException {
        log.debug("REST request to search  {}", query);
        return searchService.searchResultPage(query,SearchInputDTO.PAGE_SIZE*query.getPage(), SearchInputDTO.PAGE_SIZE);
    }

	/**
	 * returns all keyword autocompletes for keyWord
	 *
	 * @param keyWordPrefix
	 * @return
	 * @throws IOException
	 */
    @GetMapping("/search/keywordsAutoComplete")
	public List<String> getKeywordsAutoComplete(@RequestParam String keyWordPrefix) throws IOException {
		return searchService.getKeywordsAutoComplete(keyWordPrefix);
	}

	/**
	 * returns all creator autocompletes
	 *
	 * @param creatorPrefix
	 * @return
	 * @throws IOException
	 */
    @GetMapping("/search/creatorAutoComplete")
	public List<String> getCreatorAutoComplete(@RequestParam String creatorPrefix) throws IOException {
		return searchService.getCreatorAutoComplete(creatorPrefix);
	}

	/**
	 * returns all contributor autocompletes
	 *
	 * @param contributorPrefix
	 * @return
	 * @throws IOException
	 */
    @GetMapping("/search/contributorAutoComplete")
	public List<String> getContributorAutoComplete(@RequestParam String contributorPrefix) throws IOException {
		return searchService.getContributorAutoComplete(contributorPrefix);
	}

	/**
	 * returns all programmingLanguage autocompletes for keyWord
	 *
	 * @param programmingLanguagePrefix
	 * @return
	 * @throws IOException
	 */
    @GetMapping("/search/programmingLanguageAutoComplete")
	public List<String> getProgrammingLanguageAutoComplete(@RequestParam String programmingLanguagePrefix) throws IOException {
		return searchService.getProgrammingLanguageAutoComplete(programmingLanguagePrefix);
	}


}
